// SPDX-License-Identifier: MIT
pragma solidity ^0.8.22;

import "@openzeppelin/contracts-upgradeable/access/OwnableUpgradeable.sol";
import "@openzeppelin/contracts-upgradeable/proxy/utils/Initializable.sol";


contract Admin is Initializable, OwnableUpgradeable {
    mapping(address => bool) private _admins;

    modifier onlyAdmin() {
        require(_admins[msg.sender], "Only admin can call it");
        _;
    }

//    constructor() {
//        _disableInitializers();
//    }

    function admin_initialize() internal onlyInitializing {
        __Ownable_init(msg.sender); // Initialize with the owner
        _admins[msg.sender] = true; // Set the initial owner as admin
    }

    function addAdmin(address _admin) public onlyOwner {
        _admins[_admin] = true;
    }

    function delAdmin(address _admin) public onlyOwner {
        _admins[_admin] = false;
    }

    function isAdmin(address _addr) public view returns (bool) {
        return _admins[_addr];
    }
}

contract Program is Admin {
    mapping(address => bool) private _programs;

    modifier onlyProgram() {
        require(_programs[msg.sender], "Only program can call it");
        _;
    }

    function program_initialize() internal onlyInitializing {
        admin_initialize();
    }

    function addProgram(address _program) public onlyOwner {
        _programs[_program] = true;
    }

    function delProgram(address _program) public onlyOwner {
        _programs[_program] = false;
    }

    function isProgram(address _program) public view returns (bool) {
        return _programs[_program];
    }
}
